from django.db import models
from django.core.cache import cache


class Placeholder(models.Model):
    key = models.CharField(max_length=40, primary_key=True, db_index=True)
    value = models.TextField(blank=True)

    def __unicode__(self):
        return self.value

    def cache_key(self):
        return "front-edit-%s" % self.key

    def save(self, *args, **kwargs):
        super(Placeholder, self).save(*args, **kwargs)
        cache.delete(self.cache_key())



from django import template
from classytags.core import Tag, Options
from classytags.arguments import Argument, MultiValueArgument, KeywordArgument
from django.core.cache import cache
from django.core.urlresolvers import reverse
from ..models import Placeholder
from ..conf import settings as django_front_settings
import hashlib


register = template.Library()


class FrontEditTag(Tag):
    name = 'front_edit'
    options = Options(
        Argument('name', resolve=False, required=True),
        MultiValueArgument('extra_bits', required=False, resolve=True),
        blocks=[
            ('end_front_edit', 'nodelist'),
        ]
    )

    def render_tag(self, context, name, extra_bits, nodelist=None):
        hash_val = hashlib.new('sha1', name + ''.join([unicode(token) for token in extra_bits])).hexdigest()
        cache_key = "front-edit-%s" % hash_val

        val = cache.get(cache_key)
        if val is None:
            try:
                val = Placeholder.objects.get(key=hash_val).value
                cache.set(cache_key, val, 3600 * 24)
            except Placeholder.DoesNotExist:
                pass

        if val is None and nodelist:
            val = nodelist.render(context)

        user = context.get('request', None) and context.get('request').user
        if django_front_settings.DJANGO_FRONT_PERMISSION(user):
            return '<div class="editable" id="%s">%s</div>' % (hash_val, unicode(val).strip())
        return val or ''


class FrontEditJS(Tag):
    name = 'front_edit_scripts'
    options = Options(
        KeywordArgument('editor', resolve=False, required=False, defaultkey='editor')
    )

    def render_tag(self, context, editor=''):
        static_url = context.get('STATIC_URL', '/static/')
        user = context.get('request', None) and context.get('request').user
        token = unicode(context.get('csrf_token'))
        plugin = editor.get('editor').lower() if editor.get('editor') and editor.get('editor').lower() in ['ace'] else ''

        if django_front_settings.DJANGO_FRONT_PERMISSION(user):
            return """
<link rel="stylesheet" href="%s/front/css/front-edit.css" />
<script>
    document._front_edit = {
        save_url: '%s',
        csrf_token: '%s',
        plugin: '%s'
    };
</script>
<script src="%sfront/js/front-edit.js"></script>""".strip() % (
                static_url,
                reverse('front-placeholder-save'),
                token,
                plugin,
                static_url
            )
        else:
            return ''


register.tag(FrontEditTag)
register.tag(FrontEditJS)



from .models import Placeholder
from django.http import HttpResponse
from .conf import settings as django_front_settings


def do_save(request):
    if request.POST and django_front_settings.DJANGO_FRONT_PERMISSION(request.user):
        key, val = request.POST.get('key'), request.POST.get('val')
        placeholder, _ = Placeholder.objects.get_or_create(key=key)
        placeholder.value = val
        placeholder.save()

        return HttpResponse('1')
    return HttpResponse('0')



try:
    from django.conf.urls import *
except ImportError:  # django < 1.4
    from django.conf.urls.defaults import *


urlpatterns = patterns('front.views',
    url(r'^save/$', 'do_save', name='front-placeholder-save'),
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Placeholder'
        db.create_table(u'front_placeholder', (
            ('key', self.gf('django.db.models.fields.CharField')(max_length=40, primary_key=True, db_index=True)),
            ('value', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal(u'front', ['Placeholder'])


    def backwards(self, orm):
        # Deleting model 'Placeholder'
        db.delete_table(u'front_placeholder')


    models = {
        u'front.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'key': ('django.db.models.fields.CharField', [], {'max_length': '40', 'primary_key': 'True', 'db_index': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['front']


from setuptools import setup, find_packages
setup(
    name='django-front',
    version='0.1.1',
    description='A Django application to allow front-end editing',
    author='Marco Bonetti',
    author_email='mbonetti@gmail.com',
    url='https://github.com/mbi/django-front',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'django-classy-tags >= 0.4',
        'Django >= 1.4'
    ]
)
